@@ -1,6 +1,7 @@
 //! Load commands tell the kernel and dynamic linker anything from how to load this binary into memory, what the entry point is, apple specific information, to which libraries it requires for dynamic linking
 
 use crate::error;
+use core::convert::TryFrom;
 use core::fmt::{self, Display};
 use scroll::{ctx, Endian};
 use scroll::{IOread, IOwrite, Pread, Pwrite, SizeWith};
@@ -506,18 +507,6 @@ impl<'a> ctx::TryFromCtx<'a, Endian> for ThreadCommand {
         let flavor: u32 = bytes.pread_with(8, le)?;
         let count: u32 = bytes.pread_with(12, le)?;
 
-        // get a byte slice of the thread state
-        let thread_state_byte_length = count as usize * 4;
-        let thread_state_bytes = &bytes[16..16 + thread_state_byte_length];
-
-        // check the length
-        if thread_state_bytes.len() < thread_state_byte_length {
-            return Err(error::Error::Malformed(format!(
-                "thread command specifies {} bytes for thread state but has only {}",
-                thread_state_byte_length,
-                thread_state_bytes.len()
-            )));
-        }
         if count > 70 {
             return Err(error::Error::Malformed(format!(
                 "thread command specifies {} longs for thread state but we handle only 70",
@@ -525,6 +514,20 @@ impl<'a> ctx::TryFromCtx<'a, Endian> for ThreadCommand {
             )));
         }
 
+        // get a byte slice of the thread state
+        let thread_state_byte_length = count as usize * 4;
+
+        // check the length
+        if bytes.len() < 16 + thread_state_byte_length {
+            return Err(error::Error::Malformed(format!(
+                "thread command specifies {} bytes for thread state but has only {}",
+                thread_state_byte_length,
+                bytes.len()
+            )));
+        }
+
+        let thread_state_bytes = &bytes[16..16 + thread_state_byte_length];
+
         // read the thread state
         let mut thread_state: [u32; 70] = [0; 70];
         for (i, state) in thread_state.iter_mut().enumerate().take(count as usize) {
@@ -944,9 +947,10 @@ pub const SIZEOF_RPATH_COMMAND: usize = 12;
 /// The linkedit_data_command contains the offsets and sizes of a blob
 /// of data in the __LINKEDIT segment.
 #[repr(C)]
-#[derive(Debug, Clone, Copy, Pread, Pwrite, IOread, IOwrite, SizeWith)]
+#[derive(Default, Debug, Clone, Copy, Pread, Pwrite, IOread, IOwrite, SizeWith)]
 pub struct LinkeditDataCommand {
-    /// LC_CODE_SIGNATURE, LC_SEGMENT_SPLIT_INFO, LC_FUNCTION_STARTS, LC_DATA_IN_CODE, LC_DYLIB_CODE_SIGN_DRS or LC_LINKER_OPTIMIZATION_HINT.
+    /// LC_CODE_SIGNATURE, LC_SEGMENT_SPLIT_INFO, LC_FUNCTION_STARTS, LC_DATA_IN_CODE,
+    /// LC_DYLIB_CODE_SIGN_DRS, LC_LINKER_OPTIMIZATION_HINT, LC_DYLD_EXPORTS_TRIE, or LC_DYLD_CHAINED_FIXUPS.
     pub cmd: u32,
     /// sizeof(struct linkedit_data_command)
     pub cmdsize: u32,
@@ -998,13 +1002,43 @@ pub struct EncryptionInfoCommand64 {
 
 pub const SIZEOF_ENCRYPTION_INFO_COMMAND_64: usize = 24;
 
+/// An enumeration of platforms currently identifiable within a version_min_command.
+#[non_exhaustive]
+#[repr(u32)]
+#[derive(Debug)]
+pub enum Platform {
+    Macos = LC_VERSION_MIN_MACOSX,
+    Iphoneos = LC_VERSION_MIN_IPHONEOS,
+    Tvos = LC_VERSION_MIN_TVOS,
+    Watchos = LC_VERSION_MIN_WATCHOS,
+}
+
+impl TryFrom<u32> for Platform {
+    type Error = error::Error;
+
+    fn try_from(cmd: u32) -> Result<Self, Self::Error> {
+        Ok(match cmd {
+            LC_VERSION_MIN_MACOSX => Platform::Macos,
+            LC_VERSION_MIN_IPHONEOS => Platform::Iphoneos,
+            LC_VERSION_MIN_TVOS => Platform::Tvos,
+            LC_VERSION_MIN_WATCHOS => Platform::Watchos,
+            _ => {
+                return Err(error::Error::Malformed(format!(
+                    "unknown platform for load command: {:x}",
+                    cmd
+                )))
+            }
+        })
+    }
+}
+
 /// The version_min_command contains the min OS version on which this
 /// binary was built to run.
 ///
-/// LC_VERSION_MIN_MACOSX or LC_VERSION_MIN_IPHONEOS
 #[repr(C)]
 #[derive(Debug, Clone, Copy, Pread, Pwrite, IOread, IOwrite, SizeWith)]
 pub struct VersionMinCommand {
+    /// LC_VERSION_MIN_MACOSX, LC_VERSION_MIN_IPHONEOS, LC_VERSION_MIN_TVOS, or LC_VERSION_MIN_WATCHOS.
     pub cmd: u32,
     pub cmdsize: u32,
     /// X.Y.Z is encoded in nibbles xxxx.yy.zz
@@ -1014,18 +1048,22 @@ pub struct VersionMinCommand {
 }
 
 impl VersionMinCommand {
-    pub fn new(is_ios: bool) -> Self {
+    pub fn new(platform: Platform) -> Self {
         VersionMinCommand {
-            cmd: if is_ios {
-                LC_VERSION_MIN_IPHONEOS
-            } else {
-                LC_VERSION_MIN_MACOSX
-            },
+            cmd: platform as u32,
             cmdsize: SIZEOF_VERSION_MIN_COMMAND as u32,
             version: 0,
             sdk: 0,
         }
     }
+
+    pub fn platform(&self) -> Platform {
+        // A panic here indicates an incomplete API change above: VersionMinCommand
+        // can only be constructed from one of the LC_VERSION_* commands or directly
+        // from a Platform, so an error indicates that a new one hasn't been correctly
+        // added to the Platform enum.
+        Platform::try_from(self.cmd).expect("impossible platform (implementation error)")
+    }
 }
 
 pub const SIZEOF_VERSION_MIN_COMMAND: usize = 16;
@@ -1147,6 +1185,53 @@ pub struct EntryPointCommand {
 
 pub const SIZEOF_ENTRY_POINT_COMMAND: usize = 24;
 
+/// The build_version_command contains the min OS version on which this
+/// binary was built to run for its platform.  The list of known platforms and
+/// tool values following it.
+#[repr(C)]
+#[derive(Debug, Clone, Copy, Pread, Pwrite, IOread, IOwrite, SizeWith)]
+pub struct BuildVersionCommand {
+    /// LC_BUILD_VERSION
+    pub cmd: u32,
+    pub cmdsize: u32,
+    /// platform
+    pub platform: u32,
+    /// X.Y.Z is encoded in nibbles xxxx.yy.zz
+    pub minos: u32,
+    /// X.Y.Z is encoded in nibbles xxxx.yy.zz
+    pub sdk: u32,
+    /// number of tool entries following this
+    pub ntools: u32,
+}
+
+/// Build tool version
+#[repr(C)]
+#[derive(Debug, Clone, Copy, Pread, Pwrite, IOread, IOwrite, SizeWith)]
+pub struct BuildToolVersion {
+    /// enum for the tool
+    pub tool: u32,
+    /// version number of the tool
+    pub version: u32,
+}
+
+/// The LC_FILESET_ENTRY command is used for Mach-O filesets which contain
+/// multiple Mach-O's, such as the dyld shared cache and kernelcache
+#[repr(C)]
+#[derive(Debug, Clone, Copy, Pread, Pwrite, IOread, IOwrite, SizeWith)]
+pub struct FilesetEntryCommand {
+    /// LC_FILSET_ENTRY
+    pub cmd: u32,
+    pub cmdsize: u32,
+    /// memory address of the dylib
+    pub vmaddr: u64,
+    /// file offset of the dylib
+    pub fileoff: u64,
+    /// contained entry id
+    pub entry_id: LcStr,
+    /// reserved
+    pub reserved: u32,
+}
+
 /// The source_version_command is an optional load command containing
 /// the version of the sources used to build the binary.
 #[repr(C)]
@@ -1173,6 +1258,22 @@ pub struct DataInCodeEntry {
     pub kind: u16,
 }
 
+/// LC_NOTE commands describe a region of arbitrary data included in a Mach-O
+/// file.  Its initial use is to record extra data in MH_CORE files.
+#[repr(C)]
+#[derive(Debug, Clone, Copy, Pread, Pwrite, IOread, IOwrite, SizeWith)]
+pub struct NoteCommand {
+    /// LC_NOTE
+    pub cmd: u32,
+    pub cmdsize: u32,
+    /// owner name for this LC_NOTE
+    pub data_owner: [u8; 16],
+    /// file offset of this data
+    pub offset: u64,
+    /// length of data region
+    pub size: u64,
+}
+
 ///////////////////////////////////////
 // Constants, et. al
 ///////////////////////////////////////
@@ -1184,6 +1285,8 @@ pub const LC_REEXPORT_DYLIB: u32 = 0x1f | LC_REQ_DYLD;
 pub const LC_DYLD_INFO_ONLY: u32 = 0x22 | LC_REQ_DYLD;
 pub const LC_LOAD_UPWARD_DYLIB: u32 = 0x23 | LC_REQ_DYLD;
 pub const LC_MAIN: u32 = 0x28 | LC_REQ_DYLD;
+pub const LC_DYLD_EXPORTS_TRIE: u32 = 0x33 | LC_REQ_DYLD;
+pub const LC_DYLD_CHAINED_FIXUPS: u32 = 0x34 | LC_REQ_DYLD;
 pub const LC_SEGMENT: u32 = 0x1;
 pub const LC_SYMTAB: u32 = 0x2;
 pub const LC_SYMSEG: u32 = 0x3;
@@ -1225,6 +1328,27 @@ pub const LC_DYLIB_CODE_SIGN_DRS: u32 = 0x2B;
 pub const LC_ENCRYPTION_INFO_64: u32 = 0x2C;
 pub const LC_LINKER_OPTION: u32 = 0x2D;
 pub const LC_LINKER_OPTIMIZATION_HINT: u32 = 0x2E;
+pub const LC_VERSION_MIN_TVOS: u32 = 0x2F;
+pub const LC_VERSION_MIN_WATCHOS: u32 = 0x30;
+pub const LC_NOTE: u32 = 0x31;
+pub const LC_BUILD_VERSION: u32 = 0x32;
+pub const LC_FILESET_ENTRY: u32 = 0x35 | LC_REQ_DYLD;
+pub const PLATFORM_MACOS: u32 = 1;
+pub const PLATFORM_IOS: u32 = 2;
+pub const PLATFORM_TVOS: u32 = 3;
+pub const PLATFORM_WATCHOS: u32 = 4;
+pub const PLATFORM_BRIDGEOS: u32 = 5;
+pub const PLATFORM_MACCATALYST: u32 = 6;
+pub const PLATFORM_IOSSIMULATOR: u32 = 7;
+pub const PLATFORM_TVOSSIMULATOR: u32 = 8;
+pub const PLATFORM_WATCHOSSIMULATOR: u32 = 9;
+pub const PLATFORM_DRIVERKIT: u32 = 10;
+pub const PLATFORM_VISIONOS: u32 = 11;
+pub const PLATFORM_VISIONOSSIMULATOR: u32 = 12;
+pub const TOOL_CLANG: u32 = 1;
+pub const TOOL_SWIFT: u32 = 2;
+pub const TOOL_LD: u32 = 3;
+pub const TOOL_LLD: u32 = 4;
 
 pub fn cmd_to_str(cmd: u32) -> &'static str {
     match cmd {
@@ -1275,6 +1399,13 @@ pub fn cmd_to_str(cmd: u32) -> &'static str {
         LC_ENCRYPTION_INFO_64 => "LC_ENCRYPTION_INFO_64",
         LC_LINKER_OPTION => "LC_LINKER_OPTION",
         LC_LINKER_OPTIMIZATION_HINT => "LC_LINKER_OPTIMIZATION_HINT",
+        LC_VERSION_MIN_TVOS => "LC_VERSION_MIN_TVOS",
+        LC_VERSION_MIN_WATCHOS => "LC_VERSION_MIN_WATCHOS",
+        LC_NOTE => "LC_NOTE",
+        LC_BUILD_VERSION => "LC_BUILD_VERSION",
+        LC_FILESET_ENTRY => "LC_FILESET_ENTRY",
+        LC_DYLD_EXPORTS_TRIE => "LC_DYLD_EXPORTS_TRIE",
+        LC_DYLD_CHAINED_FIXUPS => "LC_DYLD_CHAINED_FIXUPS",
         _ => "LC_UNKNOWN",
     }
 }
@@ -1285,6 +1416,7 @@ pub fn cmd_to_str(cmd: u32) -> &'static str {
 
 #[derive(Debug)]
 #[allow(clippy::large_enum_variant)]
+#[non_exhaustive]
 /// The various load commands as a cast-free variant/enum
 pub enum CommandVariant {
     Segment32(SegmentCommand32),
@@ -1330,10 +1462,17 @@ pub enum CommandVariant {
     DyldEnvironment(DylinkerCommand),
     Main(EntryPointCommand),
     DataInCode(LinkeditDataCommand),
+    BuildVersion(BuildVersionCommand),
+    FilesetEntry(FilesetEntryCommand),
     SourceVersion(SourceVersionCommand),
     DylibCodeSignDrs(LinkeditDataCommand),
     LinkerOption(LinkeditDataCommand),
     LinkerOptimizationHint(LinkeditDataCommand),
+    VersionMinTvos(VersionMinCommand),
+    VersionMinWatchos(VersionMinCommand),
+    DyldExportsTrie(LinkeditDataCommand),
+    DyldChainedFixups(LinkeditDataCommand),
+    Note(NoteCommand),
     Unimplemented(LoadCommandHeader),
 }
 
@@ -1524,6 +1663,14 @@ impl<'a> ctx::TryFromCtx<'a, Endian> for CommandVariant {
                 let comm = bytes.pread_with::<LinkeditDataCommand>(0, le)?;
                 Ok((DataInCode(comm), size))
             }
+            LC_BUILD_VERSION => {
+                let comm = bytes.pread_with::<BuildVersionCommand>(0, le)?;
+                Ok((BuildVersion(comm), size))
+            }
+            LC_FILESET_ENTRY => {
+                let comm = bytes.pread_with::<FilesetEntryCommand>(0, le)?;
+                Ok((FilesetEntry(comm), size))
+            }
             LC_SOURCE_VERSION => {
                 let comm = bytes.pread_with::<SourceVersionCommand>(0, le)?;
                 Ok((SourceVersion(comm), size))
@@ -1540,6 +1687,26 @@ impl<'a> ctx::TryFromCtx<'a, Endian> for CommandVariant {
                 let comm = bytes.pread_with::<LinkeditDataCommand>(0, le)?;
                 Ok((LinkerOptimizationHint(comm), size))
             }
+            LC_VERSION_MIN_TVOS => {
+                let comm = bytes.pread_with::<VersionMinCommand>(0, le)?;
+                Ok((VersionMinTvos(comm), size))
+            }
+            LC_VERSION_MIN_WATCHOS => {
+                let comm = bytes.pread_with::<VersionMinCommand>(0, le)?;
+                Ok((VersionMinWatchos(comm), size))
+            }
+            LC_DYLD_EXPORTS_TRIE => {
+                let comm = bytes.pread_with::<LinkeditDataCommand>(0, le)?;
+                Ok((DyldExportsTrie(comm), size))
+            }
+            LC_DYLD_CHAINED_FIXUPS => {
+                let comm = bytes.pread_with::<LinkeditDataCommand>(0, le)?;
+                Ok((DyldChainedFixups(comm), size))
+            }
+            LC_NOTE => {
+                let comm = bytes.pread_with::<NoteCommand>(0, le)?;
+                Ok((Note(comm), size))
+            }
             _ => Ok((Unimplemented(lc), size)),
         }
     }
@@ -1592,10 +1759,17 @@ impl CommandVariant {
             DyldEnvironment(comm) => comm.cmdsize,
             Main(comm) => comm.cmdsize,
             DataInCode(comm) => comm.cmdsize,
+            BuildVersion(comm) => comm.cmdsize,
+            FilesetEntry(comm) => comm.cmdsize,
             SourceVersion(comm) => comm.cmdsize,
             DylibCodeSignDrs(comm) => comm.cmdsize,
             LinkerOption(comm) => comm.cmdsize,
             LinkerOptimizationHint(comm) => comm.cmdsize,
+            VersionMinTvos(comm) => comm.cmdsize,
+            VersionMinWatchos(comm) => comm.cmdsize,
+            DyldExportsTrie(comm) => comm.cmdsize,
+            DyldChainedFixups(comm) => comm.cmdsize,
+            Note(comm) => comm.cmdsize,
             Unimplemented(comm) => comm.cmdsize,
         };
         cmdsize as usize
@@ -1646,10 +1820,17 @@ impl CommandVariant {
             DyldEnvironment(comm) => comm.cmd,
             Main(comm) => comm.cmd,
             DataInCode(comm) => comm.cmd,
+            BuildVersion(comm) => comm.cmd,
+            FilesetEntry(comm) => comm.cmd,
             SourceVersion(comm) => comm.cmd,
             DylibCodeSignDrs(comm) => comm.cmd,
             LinkerOption(comm) => comm.cmd,
             LinkerOptimizationHint(comm) => comm.cmd,
+            VersionMinTvos(comm) => comm.cmd,
+            VersionMinWatchos(comm) => comm.cmd,
+            DyldExportsTrie(comm) => comm.cmd,
+            DyldChainedFixups(comm) => comm.cmd,
+            Note(comm) => comm.cmd,
             Unimplemented(comm) => comm.cmd,
         }
     }
