@@ -1,5 +1,6 @@
 use crate::error;
 use alloc::string::ToString;
+use alloc::vec::Vec;
 use scroll::Pread;
 
 use super::options;
@@ -63,6 +64,8 @@ fn section_read_size(section: &section_table::SectionTable, file_alignment: u32)
 
     if virtual_size == 0 {
         read_size
+    } else if read_size == 0 {
+        virtual_size
     } else {
         cmp::min(read_size, round_size(virtual_size))
     }
@@ -178,3 +181,18 @@ where
     let result: T = bytes.pread_with(offset, scroll::LE)?;
     Ok(result)
 }
+
+pub(crate) fn pad(length: usize, alignment: Option<usize>) -> Option<Vec<u8>> {
+    match alignment {
+        Some(alignment) => {
+            let overhang = length % alignment;
+            if overhang != 0 {
+                let repeat = alignment - overhang;
+                Some(vec![0u8; repeat])
+            } else {
+                None
+            }
+        }
+        None => None,
+    }
+}
